function [out,Xtest_out,Ftest_out,itest_out]=pref_function(x,y,f,comparetol)
% Preference query function.
%
% pref=pref_function(x,y,f,comparetol)
%
% evaluates the preference function based on f:
%
% pref = -1 if f(x) < f(y) - comparetol
%         1 if f(x) > f(y) + comparetol
%         0 if |f(x)-f(y)| <= comparetol
%
% pref_function('clear') resets values of f already computed, that are
% stored to save computations of expensive functions.
%
% val = pref_function('get',x) returns val = f(x), that is retrieved from
% existing values if the function has been already evaluated at x, or
% computes a new value if not.
%
% (C) 2019 by A. Bemporad, September 22, 2019

persistent Xtest Ftest itest

if isa(x,'char')
    if strcmp(x,'clear')
        Xtest=[];
        Ftest=[];
        itest=0;
    elseif strcmp(x,'get')
        out=pref_function_value(y);
    end
    return
end

xfound=0;
yfound=0;
for i=1:itest
    if ~xfound && sum(abs(Xtest(i,:)-x(:)'))<=1e-10
        xfound=1;
        fx=Ftest(i);
    end
    if ~yfound && sum(abs(Xtest(i,:)-y(:)'))<=1e-10
        yfound=1;
        fy=Ftest(i);
    end
end
if ~xfound
    fx=f(x(:)');
    itest=itest+1;
    if itest==1
        Xtest=x(:)';
        Ftest=fx;
    else
        Xtest(itest,:)=x(:)';
        Ftest(itest)=fx;
    end
end
if ~yfound
    fy=f(y(:)');
    itest=itest+1;
    if itest==1
        Xtest=y(:)';
        Ftest=fy;
    else
        Xtest(itest,:)=y(:)';
        Ftest(itest)=fy;
    end
end

% Make comparison
if fx<fy-comparetol
    out=-1;
elseif fx>fy+comparetol
    out=1;
else
    out=0;
end

if nargout>1
    Xtest_out=Xtest;
    Ftest_out=Ftest;
    itest_out=itest;
end

    function val=pref_function_value(x)
        % Compute function value, from available ones if available
        %
        % (C) 2019 A. Bemporad, September 22, 2019
        
        j=0;
        while j<itest
            j=j+1;
            if sum(abs(Xtest(j,:)-x(:)'))<=1e-10
                val=Ftest(j);
                return
            end
        end
        
        % Value has not been found
        val=f(x);
        itest=itest+1;
        Xtest(itest,:)=x';
        Ftest(itest)=val;
        return
    end
end